#ifndef AUDIOLAYERWIDGET_H
#define AUDIOLAYERWIDGET_H

#include <iostream>

#include <QDebug>
#include <QTime>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QSlider>
#include <QTimer>
#include <QTimeEdit>
#include <QGroupBox>
#include <QCheckBox>
#include <QSpinBox>

#include "defines.h"
#include "slidergroup.h"

class AudioLayerWidget : public QGroupBox
{
    Q_OBJECT
    
public:
    explicit AudioLayerWidget(QWidget *parent = 0, QString name = "Layer", int layer = 0);
    ~AudioLayerWidget();
    void setVol(qreal vol);
    void resume();
    void setPan(qreal pan);
    void setPitch(qreal pitch);
    void setLoop(bool on);
    void setPlaybackStatus(Status status);
    inline Status getPlaybackStatus() { return m_status; }

private:
    Status m_status;
    int m_layer;
    QPushButton *m_suspendResumeButton;
    QLabel *m_statusLabel;
    QLabel * m_statusValue;
    QLabel *m_fileLabel;
    QLabel *m_fileValue;
    QLabel * m_folderLabel;
    QLabel * m_folderValue;
    SliderGroup *m_volume;
    SliderGroup *m_pan;
    SliderGroup *m_pitch;
    QLabel * m_progressLabel;
    QSlider *m_progressSlider;
    QLabel *m_progressTimeLabel;
    QTimeEdit *m_progressTime;
    QLabel *m_totalTimeLabel;
    QTimeEdit *m_totalTimeValue;

public slots:
    void toggleSuspendResume();
    void volumeChanged(int vol);
    void panChanged(int vol);
    void pitchChanged(int vol);
    void fileLoaded(QString file);
    void durationChanged(qint64 dur);
    void refreshUi(float progress);

signals:
    void uiPlaybackChanged(int layer, Status s);
    void uiSliderChanged(int layer, Slider s, int value);

};

#endif // AUDIOLAYERWIDGET_H
