/*

    Libre Media Server Audio - An Open source Media Server for arts and performing.
    (c) Criptomart - Santiago Noreña 2012-2024 <lms@criptomart.net>
    https://git.criptomart.net/libremediaserver

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "libremediaserver-audio.h"


libreMediaServerAudio::libreMediaServerAudio(QStringList args, QWidget *parent)
    : QMainWindow(parent)
{
    Q_UNUSED(args);
    ui.setupUi(this);
    this->setWindowTitle(VERSION);
    Settings *set = Settings::getInstance();
    set->readFile();
    connect(set, SIGNAL(audioDeviceChanged(int)), this, SLOT(audioDeviceChanged(int)));
    this->setWindowTitle(VERSION);
    qDebug() << VERSION;
    qDebug() << COPYRIGHT;
    qDebug() << LICENSE;
    MediaLibrary::getInstance()->initMediaLibrary();
    aw = new AudioWidget;
    setCentralWidget(aw);
    m_dmxWidget = new dmxWidget(this);
    QDockWidget *topWidget = new QDockWidget(tr("Master"), this);
    topWidget->setAllowedAreas(Qt::TopDockWidgetArea);
    topWidget->setWidget(m_dmxWidget);
    addDockWidget(Qt::TopDockWidgetArea, topWidget);
    ola = new olaThread(this, set->getLayersNumber());
    Q_CHECK_PTR(ola);
    ola->blockSignals(true);
    connect(set, SIGNAL(registerUniverse(int)), ola, SLOT(registerUniverse(int)));
    connect(ola, SIGNAL (universeReceived(int)), m_dmxWidget, SLOT(updateWatchDMX(int)));
    connect(ola, SIGNAL(dmxOutput(int, int, int)), this, SLOT(dmxInput(int, int, int)));
    connect(ui.actionLaunch_OLA_Setup, SIGNAL(triggered()), this, SLOT(olasetup()));
    ola->registerUniverse();
    ola->start(QThread::TimeCriticalPriority );
    connect(ui.actionOpen_conf, SIGNAL(triggered()), this, SLOT(openFile()));
    connect(ui.actionSave_conf, SIGNAL(triggered()), this, SLOT(saveFile()));
    connect(ui.action_Settings, SIGNAL(triggered()), this, SLOT(settings()));
    aw->startEngine();
    qDebug("Init Complete.");
    ola->blockSignals(false);
    ola->resendDmx();
}

libreMediaServerAudio::~libreMediaServerAudio()
{
    ola->stop();
    aw->stopEngine();
}

void libreMediaServerAudio::openFile()
{
    QFileDialog dialog(this);
    if (!dialog.exec())
        return;
    QStringList fileNames;
    fileNames = dialog.selectedFiles();
    QFile file(fileNames.at(0));
//    open(&file);
}

void libreMediaServerAudio::saveFile()
{
    QFileDialog dialog(this);
    if (!dialog.exec())
        return;
    QStringList fileNames;
    fileNames = dialog.selectedFiles();
    QFile file(fileNames.at(0));
//    save(&file);
}

void libreMediaServerAudio::settings()
{
    SettingsDialog *sd = new SettingsDialog();
    sd->show();
}

void libreMediaServerAudio::olasetup()
{
    QWebView *view = new QWebView();
    view->load(QUrl("http://localhost:9090/ola.html"));
    view->show();
}

void libreMediaServerAudio::dmxInput(int layer, int channel, int value)
{
    if (layer > LAYER_CHANNELS)
        return;
    QString mediaFile = NULL;
    int aux;
    qreal f;
    int r;
    switch(channel){
    case DMX_FOLDER:
        aux = ola->getValue(layer, DMX_FILE);
        mediaFile = MediaLibrary::getInstance()->requestNewFile(value, aux);
        if (QFile::exists(mediaFile))
            aw->mediaLoaded(layer, mediaFile);
        break;
    case DMX_FILE:
        aux = ola->getValue(layer, DMX_FOLDER);
        mediaFile = MediaLibrary::getInstance()->requestNewFile(aux, value);
        if (QFile::exists(mediaFile))
            aw->mediaLoaded(layer, mediaFile);
        break;
    case VOLUME_COARSE:
        f = ( value * 0x100 ) + ola->getValue(layer, VOLUME_FINE);
        aw->volChanged(layer, (f / 655.35));
        break;
    case VOLUME_FINE:
        f = ( ola->getValue(layer, VOLUME_COARSE) * 0x100 ) + value;
        aw->volChanged(layer, (f / 655.35));
        break;
    case PAN:
        aw->panChanged(layer, value);
        break;
    case PITCH:
        aw->pitchChanged(layer, value);
        break;
    case ENTRY_POINT_COARSE:
        r = ( value * 0x100 ) + ola->getValue(layer, ENTRY_POINT_FINE);
        aw->entryPointChanged(layer, r);
        break;
    case ENTRY_POINT_FINE:
        r = ( ola->getValue(layer, ENTRY_POINT_COARSE) * 0x100 ) + value;
        aw->entryPointChanged(layer, r);
        break;
    case PLAYBACK:
        if (value == 0)
            break;
        aux = value / 25;
        switch (aux) {
        case 0 :
            aw->playbackChanged(layer, PlayingOnce);
            break;
        case 1 :
            aw->playbackChanged(layer, Stopped);
            break;
        case 2 :
            aw->playbackChanged(layer, Paused);
            break;
        case 3 :
            aw->playbackChanged(layer, PlayingLoop);
            break;
        default :
            break;
        }
    default:
        break;
    }
}

void libreMediaServerAudio::audioDeviceChanged(int id)
{
     aw->stopEngine();
     aw->startEngine(id);
}
