#ifndef SETTINGS_H
#define SETTINGS_H

#include <QObject>
#include <QXmlStreamReader>
#include <QFile>
#include <QMessageBox>
#include <QSet>

#include "medialibrary.h"
#include "audiowidget.h"
#include "defines.h"

class Settings : public QObject
{
    Q_OBJECT

public:
    static Settings *getInstance();
    inline QSet<int> getUniverses() { return m_universe; }
    inline QString getPathMedia() { return m_pathmedia; }
    void setPathMedia(QString path);
    inline QList<dmxSetting> getDmxSettings() { return m_settings; }
    inline int getLayersNumber() { return m_layersNumber; }
    inline int getUniverseNumber() { return m_universe.size(); }
    void readFile();
    void changeLayerSetup(int layer, int universe, int address);
    void removeLayer(int layer);
    void addLayer();
    inline void setLayersNumber(int layersNumber)
    {
       if (layersNumber <= MAX_LAYERS)
           m_layersNumber = layersNumber;
       else
           m_layersNumber = MAX_LAYERS;
    }
    inline int getAudioDeviceId() { return m_audioDeviceId; }
    inline void setAudioDeviceId(int id) { m_audioDeviceId = id; }

private:
   static Settings *_instance;
   QList<dmxSetting> m_settings;
   QString m_pathmedia;
   uint m_audioDeviceId;
   QSet<int> m_universe;
   int m_layersNumber;

   explicit Settings(QObject *parent = 0);
   void readFromFile(QString file);
//    void writeFile(QString filename);    // Not implemented yet
//    void writeFile();    // Not implemented yet

signals:
    void pathChanged(QString path);
    void layersNumber(int number);
    void registerUniverse(int universe);
    void audioDeviceChanged(int id);
};

#endif // SETTINGS_H
